@@ -5,6 +5,7 @@ import apt
 import apt_pkg
 import os
 import os.path
+import sys
 import macros
 
 from xml.etree.ElementTree import ElementTree
@@ -19,6 +20,11 @@ class LanguagePackageStatus(object):
         self.installed = False
         self.doChange = False
 
+    def __str__(self):
+        return 'LanguagePackageStatus(langcode: %s, pkgname %s, available: %s, installed: %s, doChange: %s' % (
+                self.languageCode, self.pkgname_template, str(self.available),
+                str(self.installed), str(self.doChange))
+
 # the language-support information
 class LanguageInformation(object):
     def __init__(self, cache, languageCode=None, language=None):
@@ -32,9 +38,10 @@ class LanguageInformation(object):
         # langPack/support status 
         self.languagePkgList = {}
         self.languagePkgList["languagePack"] = LanguagePackageStatus(languageCode, "language-pack-%s")
-        self.languagePkgList["languageSupportWritingAids"] = LanguagePackageStatus(languageCode, "language-support-writing-%s")
-        self.languagePkgList["languageSupportInputMethods"] = LanguagePackageStatus(languageCode, "language-support-input-%s")
-        self.languagePkgList["languageSupportFonts"] = LanguagePackageStatus(languageCode, "language-support-fonts-%s")
+        if not hasattr(sys, 'argv') or 'gnome-' not in sys.argv[0]:
+            self.languagePkgList["languageSupportWritingAids"] = LanguagePackageStatus(languageCode, "language-support-writing-%s")
+            self.languagePkgList["languageSupportInputMethods"] = LanguagePackageStatus(languageCode, "language-support-input-%s")
+            self.languagePkgList["languageSupportFonts"] = LanguagePackageStatus(languageCode, "language-support-fonts-%s")
         for langpkg_status in self.languagePkgList.itervalues():
             pkgname = langpkg_status.pkgname_template % languageCode
             langpkg_status.available = pkgname in cache
@@ -125,9 +132,9 @@ class LanguageSelectorPkgCache(apt.Cache):
                 (c, lc, k, v) = l.split(':')
             except ValueError:
                 continue
-            if (c == 'tr' and lc == ''):
+            if (c in ['tr', 'wa'] and lc == ''):
                 filter_list[v] = k
-            elif (c == 'wa' and lc != ''):
+            elif (c in ['wa', 'fn', 'im'] and lc != ''):
                 if '|' in lc:
                     if not v in self.multilang:
                         self.multilang[v] = []
@@ -227,10 +234,7 @@ class LanguageSelectorPkgCache(apt.Cache):
     def _getPkgList(self, languageCode):
         """ helper that returns the list of needed pkgs for the language """
         # normal langpack+support first
-        pkg_list = ["language-support-input-%s"%languageCode,\
-                    "language-support-writing-%s"%languageCode,\
-                    "language-support-fonts-%s"%languageCode,\
-                    "language-pack-%s"%languageCode]
+        pkg_list = ["language-pack-%s"%languageCode]
         # see what additional pkgs are needed
         #for (pkg, translation) in self.pkg_translations[languageCode]:
         #    if pkg in self and self[pkg].is_installed:
@@ -255,7 +259,6 @@ class LanguageSelectorPkgCache(apt.Cache):
                     #        LanguagePackageInformation or somesuch
                     if key == "languagePack":
                         self._mark_additional_translation_packages(item)
-                    if key == "languageSupportWritingAids":
                         self._mark_additional_writing_aids(item)
                 except SystemError:
                     raise ExceptionPkgCacheBroken()
@@ -306,10 +309,11 @@ class LanguageSelectorPkgCache(apt.Cache):
                                 self[pull_pkg].mark_install()
                                 #print ("Will pull: %s" % pull_pkg)
                     else:
-                        if pkg in self and \
+                        # pkg might be empty for installing unconditionally (i. e. no dependency)
+                        if (pkg == '' or (pkg in self and \
                            (self[pkg].is_installed or \
                            self[pkg].marked_install or \
-                           self[pkg].marked_upgrade) and \
+                           self[pkg].marked_upgrade))) and \
                            ((not self[pull_pkg].is_installed and \
                            not self[pull_pkg].marked_install and \
                            not self[pull_pkg].marked_upgrade) or \
@@ -322,17 +326,6 @@ class LanguageSelectorPkgCache(apt.Cache):
                         continue
                     lcount = 0
                     pcount = 0
-                    if pull_pkg in self.multilang:
-                        # package serves multiple languages.
-                        # check if other languages on this system still need this package.
-                        for l in self.multilang[pull_pkg]:
-                            p = "language-support-writing-%s" % l
-                            if p in self and \
-                               (self[p].is_installed or \
-                               self[p].marked_install or \
-                               self[p].marked_upgrade) and \
-                               not self[p].marked_delete:
-                                lcount = lcount+1
                     if '|' in pkg:
                         # multiple dependencies, if at least one of them is installed, keep the pull_pkg
                         # only remove pull_pkg if none of the dependencies are installed anymore
@@ -380,8 +373,6 @@ class LanguageSelectorPkgCache(apt.Cache):
         for (code, lang) in self._localeinfo._lang.items():
             if code == 'zh':
                 continue
-            if code == 'no':
-                continue # hack for LP#773009
             li = LanguageInformation(self, code, lang)
             if [s for s in li.languagePkgList.itervalues() if s.available]:
                 res.append(li)
