@@ -7,6 +7,10 @@ import re
 import subprocess
 import gettext
 import os
+import pwd
+import sys
+import dbus
+import warnings
 import macros
 
 from gettext import gettext as _
@@ -216,27 +220,16 @@ class LocaleInfo(object):
 
     def getUserDefaultLanguage(self):
         """
-        Reads '/var/cache/gdm/[USER]/dmrc' and '~/.profile' if present and '~/.dmrc' otherwise
-        or if some value wasn't set.
+        Reads '~/.profile' if present; otherwise - or if '~/.profile' doesn't set any
+        values - AccountsService is queried.
         Scans for LANG and LANGUAGE variable settings and returns a list [LANG, LANGUAGE].
-        In case of '~/.dmrc', we may only have the locale as a value, not the full LANGUAGE
-        variable compatible string. Therefore we generate one from the provided locale.
+        In the case of AccountsService, we only have one language, not the full LANGUAGE
+        variable compatible string. Therefore we generate one from the provided language.
         Likewise, if LANGUAGE is not defined, generate a string from the provided LANG value.
         """
         lang = ''
         language = ''
         result = []
-        if os.path.exists('/var/run/gdm.pid') and os.path.exists('/etc/init.d/gdm'):
-            is_gdm = True
-        else:
-            is_gdm = False
-        if is_gdm and 'USER' in os.environ:
-            fname = '/var/cache/gdm/%s/dmrc' % os.environ['USER']
-            if os.path.exists(fname) and os.access(fname, os.R_OK):
-                for line in open(fname):
-                    match = re.match(r'Langlist=(.*)$',line)
-                    if match:
-                        language = match.group(1)
         fname = os.path.expanduser("~/.profile")
         if os.path.exists(fname) and \
            os.access(fname, os.R_OK):
@@ -244,26 +237,29 @@ class LocaleInfo(object):
                 match_lang = re.match(r'export LANG=(.*)$',line)
                 if match_lang:
                     lang = match_lang.group(1).strip('"')
-                if len(language) == 0:
-                    match_language = re.match(r'export LANGUAGE=(.*)$',line)
-                    if match_language:
-                        language = match_language.group(1).strip('"')
-        if len(language) == 0 and is_gdm:
-            fname = os.path.expanduser("~/.dmrc")
-            if os.path.exists(fname) and \
-           	os.access(fname, os.R_OK):
-                for line in open(fname):
-                    match = re.match(r'Language=(.*)$',line)
-                    if match:
-                        language = self.makeEnvString( match.group(1) )
-        if len(lang) == 0 and not is_gdm:
-            fname = os.path.expanduser("~/.dmrc")
-            if os.path.exists(fname) and \
-           	os.access(fname, os.R_OK):
-                for line in open(fname):
-                    match = re.match(r'Language=(.*)$',line)
-                    if match:
-                        lang = match.group(1)
+                match_language = re.match(r'export LANGUAGE=(.*)$',line)
+                if match_language:
+                    language = match_language.group(1).strip('"')
+        if len(language) == 0:
+            bus = dbus.SystemBus()
+            if 'fontconfig-voodoo' in sys.argv[0] and os.getenv('SUDO_USER'):
+                # handle 'sudo fontconfig-voodoo --auto' correctly
+                user_name = os.environ['SUDO_USER']
+            else:
+                user_name = pwd.getpwuid(os.geteuid()).pw_name
+            try:
+                obj = bus.get_object('org.freedesktop.Accounts', '/org/freedesktop/Accounts')
+                iface = dbus.Interface(obj, dbus_interface='org.freedesktop.Accounts')
+                user_path = iface.FindUserByName(user_name)
+
+                obj = bus.get_object('org.freedesktop.Accounts', user_path)
+                iface = dbus.Interface(obj, dbus_interface='org.freedesktop.DBus.Properties')
+                firstLanguage = iface.Get('org.freedesktop.Accounts.User', 'Language')
+                language = self.makeEnvString(firstLanguage)
+            except Exception as msg:
+                # a failure here shouldn't trigger a fatal error
+                warnings.warn(msg.args[0].encode('UTF-8'))
+                pass
         if len(lang) == 0 and "LANG" in os.environ:
             lang = os.environ["LANG"]
         if len(lang) > 0:
@@ -311,6 +307,33 @@ class LocaleInfo(object):
         result.append(language)
         return result
 
+    def isCompleteSystemLanguage(self):
+        if not os.access(self.environments[0], os.R_OK):
+            return False
+        language_vars = {}
+        for line in open(self.environments[0]):
+            for var in 'LANGUAGE', 'LC_MESSAGES', 'LC_CTYPE', 'LC_COLLATE':
+                if line.startswith("%s=" % var):
+                    language_vars[var] = 1
+        if len(language_vars) < 4:
+            return False
+        return True
+
+    def isCompleteUserLanguage(self):
+        fname = os.path.expanduser('~/.profile')
+        if not os.access(fname, os.R_OK):
+            return False
+        language_vars = {}
+        for line in open(fname):
+            if not line.startswith('export'):
+                continue
+            for var in 'LANGUAGE', 'LC_MESSAGES', 'LC_CTYPE', 'LC_COLLATE':
+                if line.startswith('export %s=' % var):
+                    language_vars[var] = 1
+        if len(language_vars) < 4:
+            return False
+        return True
+
 
 if __name__ == "__main__":
     datadir = "/usr/share/language-selector/"
