@@ -3,53 +3,42 @@
 #
 # Released under the GPL
 #
-import apt
-import apt_pkg
-
-import aptdaemon.client
-from aptdaemon.defer import inline_callbacks
-from aptdaemon.enums import *
-from aptdaemon.gtkwidgets import AptErrorDialog, \
-                                 AptProgressDialog
-
-import pygtk
-pygtk.require('2.0')
-import gtk
-import gtk.gdk
-import pango
-import gobject
 
 import gettext
 import grp
 import locale
 import os
 import pwd
-
-import time
+import re
 import shutil
 import string
 import subprocess
 import sys
-
 import thread
 import time
 import tempfile
-
 from gettext import gettext as _
 
-from LanguageSelector.gtk.SimpleGtkbuilderApp import SimpleGtkbuilderApp
+from gi.repository import GObject, Gdk, Gtk, Pango
+
+import apt
+import apt_pkg
+
+import aptdaemon.client
+from defer import inline_callbacks
+from aptdaemon.enums import *
+from aptdaemon.gtk3widgets import AptProgressDialog
+
 from LanguageSelector.LocaleInfo import LocaleInfo
 from LanguageSelector.LanguageSelector import *
 from LanguageSelector.ImSwitch import ImSwitch
 from LanguageSelector.macros import *
+from LanguageSelector.utils import language2locale
  
 (LIST_LANG,                     # language (i18n/human-readable)
  LIST_LANG_INFO                 # the short country code (e.g. de, pt_BR)
  ) = range(2)
 
-(COMBO_LANGUAGE,
- COMBO_CODE) = range(2)
-
 (LANGTREEVIEW_LANGUAGE,
  LANGTREEVIEW_CODE) = range(2)
  
@@ -110,8 +99,9 @@ class GtkProgress(apt.progress.base.OpProgress):
         self._window = host_window
         self._progressbar = progressbar
         self._window.realize()
-        host_window.window.set_functions(gtk.gdk.FUNC_MOVE)
+        host_window.get_window().set_functions(Gdk.WMFunction.MOVE)
         self._window.set_transient_for(parent)
+
     def update(self, percent):
         #print percent
         #print self.Op
@@ -129,21 +119,24 @@ class GtkProgress(apt.progress.base.OpProgress):
         progress = self.base + percent/100 * (self.next - self.base)
         self.old = percent
         self._progressbar.set_fraction(progress/100.0)
-        while gtk.events_pending():
-            gtk.main_iteration()
+        while Gtk.events_pending():
+            Gtk.main_iteration()
     def done(self):
         self._parent.set_sensitive(True)
     def hide(self):
         self._window.hide()
 
-class GtkLanguageSelector(LanguageSelectorBase,  SimpleGtkbuilderApp):
+class GtkLanguageSelector(LanguageSelectorBase):
 
     def __init__(self, datadir, options):
         LanguageSelectorBase.__init__(self, datadir)
-        SimpleGtkbuilderApp.__init__(self,
-                                datadir+"/data/LanguageSelector.ui")
-
         self._datadir = datadir
+
+        self.widgets = Gtk.Builder()
+        self.widgets.set_translation_domain('language-selector')
+        self.widgets.add_from_file(datadir+"/data/LanguageSelector.ui")
+        self.widgets.connect_signals(self)
+
         self.is_admin = (os.getuid() == 0 or
                          grp.getgrnam("admin")[2] in os.getgroups())
 
@@ -159,16 +152,16 @@ class GtkLanguageSelector(LanguageSelectorBase,  SimpleGtkbuilderApp):
         
         #build the comboboxes (with model)
         combo = self.combobox_locale_chooser
-        model = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING)
-        cell = gtk.CellRendererText()
+        model = Gtk.ListStore(GObject.TYPE_STRING, GObject.TYPE_STRING)
+        cell = Gtk.CellRendererText()
         combo.pack_start(cell, True)
-        combo.add_attribute(cell, 'text', COMBO_LANGUAGE)
+        combo.add_attribute(cell, 'text', LANGTREEVIEW_LANGUAGE)
         combo.set_model(model)
 #        self.combo_syslang_dirty = False
 
 #        combo = self.combobox_user_language
-#        model = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING)
-#        cell = gtk.CellRendererText()
+#        model = Gtk.ListStore(GObject.TYPE_STRING, GObject.TYPE_STRING)
+#        cell = Gtk.CellRendererText()
 #        combo.pack_start(cell, True)
 #        combo.add_attribute(cell, 'text', COMBO_LANGUAGE)
 #        combo.set_model(model)
@@ -179,8 +172,8 @@ class GtkLanguageSelector(LanguageSelectorBase,  SimpleGtkbuilderApp):
         self.ac = aptdaemon.client.AptClient()
 
         combo = self.combobox_input_method
-        model = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING)
-        cell = gtk.CellRendererText()
+        model = Gtk.ListStore(GObject.TYPE_STRING, GObject.TYPE_STRING)
+        cell = Gtk.CellRendererText()
         combo.pack_start(cell, True)
         combo.add_attribute(cell, 'text', IM_NAME)
         combo.set_model(model)
@@ -220,20 +213,20 @@ class GtkLanguageSelector(LanguageSelectorBase,  SimpleGtkbuilderApp):
         if self.is_admin:
             # check if the package list is up-to-date
             if not self._cache.havePackageLists:
-                d = gtk.MessageDialog(parent=self.window_main,
-                                      flags=gtk.DIALOG_MODAL,
-                                      type=gtk.MESSAGE_INFO,
-                                      buttons=gtk.BUTTONS_CANCEL)
+                d = Gtk.MessageDialog(parent=self.window_main,
+                                      flags=Gtk.DialogFlags.MODAL,
+                                      type=Gtk.MessageType.INFO,
+                                      buttons=Gtk.ButtonsType.CANCEL)
                 d.set_markup("<big><b>%s</b></big>\n\n%s" % (
                     _("No language information available"),
                     _("The system does not have information about the "
                       "available languages yet. Do you want to perform "
                       "a network update to get them now? ")))
                 d.set_title=("")
-                d.add_button(_("_Update"), gtk.RESPONSE_YES)
+                d.add_button(_("_Update"), Gtk.ResponseType.YES)
                 res = d.run()
                 d.destroy()
-                if res == gtk.RESPONSE_YES:
+                if res == Gtk.ResponseType.YES:
                     self.setSensitive(False)
                     self.update()
                     self.updateLanguageView()
@@ -247,15 +240,26 @@ class GtkLanguageSelector(LanguageSelectorBase,  SimpleGtkbuilderApp):
             self.combobox_input_method.set_sensitive(False)
         self.setSensitive(True)
 
+    def __getattr__(self, name):
+        '''Convenient access to GtkBuilder objects'''
+
+        o = self.widgets.get_object(name)
+        if o is None:
+            raise AttributeError, 'No such widget: ' + name
+        return o
+
+    def run(self):
+        Gtk.main()
+
     def setSensitive(self, value):
         if value:
             self.window_main.set_sensitive(True)
-            self.window_main.window.set_cursor(None)
+            self.window_main.get_window().set_cursor(None)
         else:
             self.window_main.set_sensitive(False)
-            self.window_main.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.WATCH))
-        while gtk.events_pending():
-            gtk.main_iteration()
+            self.window_main.get_window().set_cursor(Gdk.Cursor.new(Gdk.CursorType.WATCH))
+        while Gtk.events_pending():
+            Gtk.main_iteration()
 
 #    @blockSignals
 #    def updateSyncButton(self):
@@ -282,7 +286,7 @@ class GtkLanguageSelector(LanguageSelectorBase,  SimpleGtkbuilderApp):
 
     def setupInstallerTreeView(self):
         """ do all the treeview setup here """
-        def toggle_cell_func(column, cell, model, iter):
+        def toggle_cell_func(column, cell, model, iter, data):
             langInfo = model.get_value(iter, LIST_LANG_INFO)
 
             # check for active and inconsitent 
@@ -293,7 +297,7 @@ class GtkLanguageSelector(LanguageSelectorBase,  SimpleGtkbuilderApp):
             cell.set_property("active", langInfo.fullInstalled)
             cell.set_property("inconsistent", inconsistent)
             
-        def lang_view_func(cell_layout, renderer, model, iter):
+        def lang_view_func(cell_layout, renderer, model, iter, data):
             langInfo = model.get_value(iter, LIST_LANG_INFO)
             langName = model.get_value(iter, LIST_LANG)
             inconsistent = langInfo.inconsistent
@@ -303,24 +307,24 @@ class GtkLanguageSelector(LanguageSelectorBase,  SimpleGtkbuilderApp):
                 markup = "%s" % langName
             renderer.set_property("markup", markup)
 
-        renderer = gtk.CellRendererText()
-        column = gtk.TreeViewColumn(_("Language"), renderer, text=LIST_LANG)
+        renderer = Gtk.CellRendererText()
+        column = Gtk.TreeViewColumn(_("Language"), renderer, text=LIST_LANG)
         column.set_property("expand", True)
-        column.set_cell_data_func (renderer, lang_view_func)
+        column.set_cell_data_func (renderer, lang_view_func, None)
         self.treeview_languages.append_column(column)
 
-        renderer= gtk.CellRendererToggle()
+        renderer= Gtk.CellRendererToggle()
         renderer.connect("toggled", self.on_toggled)
-        column = gtk.TreeViewColumn(_("Installed"), renderer)
-        column.set_cell_data_func (renderer, toggle_cell_func)
+        column = Gtk.TreeViewColumn(_("Installed"), renderer)
+        column.set_cell_data_func (renderer, toggle_cell_func, None)
         self.treeview_languages.append_column(column)
         # build the store
-        self._langlist = gtk.ListStore(str, gobject.TYPE_PYOBJECT)
+        self._langlist = Gtk.ListStore(str, GObject.TYPE_PYOBJECT)
         self.treeview_languages.set_model(self._langlist)
 
     def setupLanguageTreeView(self):
         """ do all the treeview setup here """
-        def lang_view_func(cell_layout, renderer, model, iter):
+        def lang_view_func(cell_layout, renderer, model, iter, data):
             langInfo = model.get_value(iter, LANGTREEVIEW_CODE)
             langName = model.get_value(iter, LANGTREEVIEW_LANGUAGE)
             greyFlag = False
@@ -341,15 +345,15 @@ class GtkLanguageSelector(LanguageSelectorBase,  SimpleGtkbuilderApp):
                 markup = "%s" % self._localeinfo.translate(langInfo, native=True, allCountries=True)
             renderer.set_property("markup", markup)
 
-        renderer = gtk.CellRendererText()
-        column = gtk.TreeViewColumn(_("Language"), renderer, text=COMBO_LANGUAGE)
+        renderer = Gtk.CellRendererText()
+        column = Gtk.TreeViewColumn(_("Language"), renderer, text=LANGTREEVIEW_LANGUAGE)
         column.set_property("expand", True)
-        column.set_cell_data_func (renderer, lang_view_func)
+        column.set_cell_data_func (renderer, lang_view_func, None)
         self.treeview_locales.append_column(column)
 
         # build the store
-        self._localelist = gtk.ListStore(str, gobject.TYPE_PYOBJECT)
-        self.treeview_locales.set_model(self._localelist)
+        self._language_options = Gtk.ListStore(GObject.TYPE_STRING, GObject.TYPE_STRING)
+        self.treeview_locales.set_model(self._language_options)
 
     def _get_langinfo_on_cursor(self):
         (path, column) = self.treeview_languages.get_cursor()
@@ -431,34 +435,31 @@ class GtkLanguageSelector(LanguageSelectorBase,  SimpleGtkbuilderApp):
         """
         if not self.imSwitch.available():
             return
-        # get current selected default language
-        combo = self.combobox_locale_chooser
-        model = combo.get_model()
-        if combo.get_active() < 0:
-            return
-        (lang, code) = model[combo.get_active()]
-        #print "Active language: "+code
+        # get the current first item in the user LANGUAGE list,
+        # but with country code added if not already present
+        locale = language2locale(self.userEnvLanguage)
+        code = re.split('[.@]', locale)[0]
 
         combo = self.combobox_input_method
         #cell = combo.get_child().get_cell_renderers()[0]
         # FIXME: use something else than a hardcoded value here
         #cell.set_property("wrap-width",300)
-        #cell.set_property("wrap-mode",pango.WRAP_WORD)
+        #cell.set_property("wrap-mode",Pango.WRAP_WORD)
         model = combo.get_model()
         model.clear()
 
         # find the default
         currentIM = self.imSwitch.getInputMethodForLocale(code)
-        if currentIM == None:
+        if currentIM in (None, 'default'):
             currentIM = 'none'
         #print "Current IM: "+currentIM
 
         # find out about the other options
         for (i, IM) in enumerate(self.imSwitch.getAvailableInputMethods()):
             iter = model.append()
-            model.set(iter,
-                      IM_CHOICE,IM,
-                      IM_NAME, IM)
+            name = _('none') if IM == 'none' else IM
+            model.set_value(iter, IM_CHOICE, IM)
+            model.set_value(iter, IM_NAME, name)
             if IM == currentIM:
                 combo.set_active(i)
 #        self.check_status()
@@ -519,10 +520,10 @@ class GtkLanguageSelector(LanguageSelectorBase,  SimpleGtkbuilderApp):
         return (to_inst, to_rm)
 
     def error(self, summary, msg):
-        d = gtk.MessageDialog(parent=self.window_main,
-                              flags=gtk.DIALOG_MODAL,
-                              type=gtk.MESSAGE_ERROR,
-                              buttons=gtk.BUTTONS_CLOSE)
+        d = Gtk.MessageDialog(parent=self.window_main,
+                              flags=Gtk.DialogFlags.MODAL,
+                              type=Gtk.MessageType.ERROR,
+                              buttons=Gtk.ButtonsType.CLOSE)
         d.set_markup("<big><b>%s</b></big>\n\n%s" % (summary, msg))
         d.set_title=("")
         res = d.run()
@@ -580,7 +581,9 @@ class GtkLanguageSelector(LanguageSelectorBase,  SimpleGtkbuilderApp):
                 _("Could not install the full language support"),
                 _("Usually this is related to an error in your "
                   "software archive or software manager. Check your "
-                  "software preferences in the System > Administration menu."))
+                  "preferences in Software Sources (click the icon "
+                  "at the very right of the top bar and select "
+                  "\"System Settings... -> Software Sources\")."))
             self.setSensitive(True)
             return 0
         #print "inst_list: %s " % inst_list
@@ -598,8 +601,8 @@ class GtkLanguageSelector(LanguageSelectorBase,  SimpleGtkbuilderApp):
         
     def _wait_for_aptdaemon_finish(self):
         while not self._transaction_finished:
-            while gtk.events_pending():
-                gtk.main_iteration()
+            while Gtk.events_pending():
+                Gtk.main_iteration()
             time.sleep(0.02)
 
     def _on_finished(self, dialog):
@@ -631,7 +634,7 @@ class GtkLanguageSelector(LanguageSelectorBase,  SimpleGtkbuilderApp):
         try:
             trans = yield self.ac.commit_packages(
                 install=inst, reinstall=[], remove=rm, purge=[], upgrade=[],
-                defer=True)
+                downgrade=[], defer=True)
             self._run_transaction(trans)
         except Exception, e:
             self._show_error_dialog(e)
@@ -641,7 +644,7 @@ class GtkLanguageSelector(LanguageSelectorBase,  SimpleGtkbuilderApp):
     commit = commit_aptdaemon
 
     def hide_on_delete(self, widget, event):
-        return gtk.Widget.hide_on_delete(widget)
+        return Gtk.Widget.hide_on_delete(widget)
 
     def verifyInstalledLangPacks(self):
         """ called at the start to inform about possible missing
@@ -653,37 +656,37 @@ class GtkLanguageSelector(LanguageSelectorBase,  SimpleGtkbuilderApp):
         #print "Missing: %s " % missing
         if len(missing) > 0:
             # FIXME: add "details"
-            d = gtk.MessageDialog(parent=self.window_main,
-                                  flags=gtk.DIALOG_MODAL,
-                                  type=gtk.MESSAGE_QUESTION)
+            d = Gtk.MessageDialog(parent=self.window_main,
+                                  flags=Gtk.DialogFlags.MODAL,
+                                  type=Gtk.MessageType.QUESTION)
             d.set_markup("<big><b>%s</b></big>\n\n%s" % (
                 _("The language support is not installed completely"),
                 _("Some translations or writing aids available for your "
                   "chosen languages are not installed yet. Do you want "
                   "to install them now?")))
-            d.add_buttons(_("_Remind Me Later"), gtk.RESPONSE_NO,
-                          _("_Install"), gtk.RESPONSE_YES)
-            d.set_default_response(gtk.RESPONSE_YES)
+            d.add_buttons(_("_Remind Me Later"), Gtk.ResponseType.NO,
+                          _("_Install"), Gtk.ResponseType.YES)
+            d.set_default_response(Gtk.ResponseType.YES)
             d.set_title("")
-            expander = gtk.Expander(_("Details"))
-            scroll = gtk.ScrolledWindow()
-            scroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
-            textview = gtk.TextView()
+            expander = Gtk.Expander.new(_("Details"))
+            scroll = Gtk.ScrolledWindow()
+            scroll.set_policy(Gtk.PolicyType.AUTOMATIC, Gtk.PolicyType.AUTOMATIC)
+            textview = Gtk.TextView()
             textview.set_cursor_visible(False)
             textview.set_editable(False)
             buf = textview.get_buffer()
             pkgs = ""
             for pkg in missing:
                 pkgs += "%s\n" % pkg
-            buf.set_text(pkgs)
+            buf.set_text(pkgs, -1)
             buf.place_cursor(buf.get_start_iter())
             expander.add(scroll)
             scroll.add(textview)
-            d.vbox.pack_start(expander)
+            d.get_message_area().pack_start(expander, True, True, 0)
             expander.show_all()
             res = d.run()
             d.destroy()
-            if res == gtk.RESPONSE_YES:
+            if res == Gtk.ResponseType.YES:
                 self.setSensitive(False)
                 self.commit(missing, [])
                 self.updateLanguageView()
@@ -716,18 +719,20 @@ class GtkLanguageSelector(LanguageSelectorBase,  SimpleGtkbuilderApp):
             #if inconsistent:
             #    print "inconsistent", lang.language
             installed = lang.fullInstalled
-            lang_name = self._localeinfo.translate(lang.languageCode)
+
+            # hack for Vietnamese users; see https://launchpad.net/bugs/783090
+            # Even if calling self._localeinfo.translate() with native=True triggers
+            # a UnicodeWarning for Norwegian Bokmaal, the output should be correct.
+            lang_name = None
+            if 'LANGUAGE' in os.environ:
+                current_language = os.environ['LANGUAGE']
+                if re.match('vi[^a-z]', current_language):
+                    lang_name = self._localeinfo.translate(lang.languageCode, native=True)
+            if not lang_name:
+                lang_name = self._localeinfo.translate(lang.languageCode)
+
             self._langlist.append([lang_name, lang])
-        self._langlist.set_sort_column_id(LIST_LANG, gtk.SORT_ASCENDING)
-        for button in ( 
-              "checkbutton_translations",
-              "checkbutton_writing_aids",
-              "checkbutton_input_methods",
-              "checkbutton_fonts"):
-            self.block_toggle = True
-            getattr(self, button).set_active(False)
-            getattr(self, button).set_sensitive(False)
-            self.block_toggle = False
+        self._langlist.set_sort_column_id(LIST_LANG, Gtk.SortType.ASCENDING)
 
     def writeSystemDefaultLang(self):
         combo = self.combobox_locale_chooser
@@ -735,12 +740,17 @@ class GtkLanguageSelector(LanguageSelectorBase,  SimpleGtkbuilderApp):
         if combo.get_active() < 0:
             return False
         (lang, code) = model[combo.get_active()]
-        old_code = self._localeinfo.getSystemDefaultLanguage()[0]
+        (old_code, language_string) = self._localeinfo.getSystemDefaultLanguage()
         # no changes, nothing to do
         macr = macros.LangpackMacros(self._datadir, old_code)
         if macr["LOCALE"] == code:
             return False
         self.writeSysLangSetting(sysLang=code)
+        if self._localeinfo.isCompleteSystemLanguage():
+            return True
+        # set the system language completely in order to prevent surprises
+        # (the installer does not set system LC_* variables)
+        self.writeSysLanguageSetting(sysLanguage=language_string)
         return True
 
     def writeUserDefaultLang(self):
@@ -749,7 +759,7 @@ class GtkLanguageSelector(LanguageSelectorBase,  SimpleGtkbuilderApp):
         if combo.get_active() < 0:
             return
         (lang, code) = model[combo.get_active()]
-        temp = self._localeinfo.getUserDefaultLanguage()[0]
+        (temp, languageString) = self._localeinfo.getUserDefaultLanguage()
         if temp == None:
             old_code = self._localeinfo.getSystemDefaultLanguage()[0]
         else:
@@ -759,6 +769,12 @@ class GtkLanguageSelector(LanguageSelectorBase,  SimpleGtkbuilderApp):
         if macr["LOCALE"] == code:
             return False
         self.writeUserLangSetting(userLang=code)
+        if self._localeinfo.isCompleteUserLanguage():
+            return True
+        # set the user language completely in order to prevent surprises
+        # (the installer does not set the user language for the user that
+        # is created at installation)
+        self.writeUserLanguageSetting(userLanguage=languageString)
         return True
 
     def writeSystemLanguage(self, languageString):
@@ -785,10 +801,11 @@ class GtkLanguageSelector(LanguageSelectorBase,  SimpleGtkbuilderApp):
     def updateLocaleChooserCombo(self):
         #print "updateLocaleChooserCombo()"
         combo = self.combobox_locale_chooser
-        cell = combo.get_child().get_cell_renderers()[0]
+        #XXX get_cell_renderers does not exist in GTK3
+        #cell = combo.get_child().get_cell_renderers()[0]
         # FIXME: use something else than a hardcoded value here
-        cell.set_property("wrap-width",300)
-        cell.set_property("wrap-mode",pango.WRAP_WORD)
+        #cell.set_property("wrap-width",300)
+        #cell.set_property("wrap-mode",Pango.WRAP_WORD)
         model = combo.get_model()
         model.clear()
 
@@ -804,26 +821,12 @@ class GtkLanguageSelector(LanguageSelectorBase,  SimpleGtkbuilderApp):
 
         # find out about the other options        
 
-        self._localelist.clear()
-        installedLangList = []
+        """ languages for message translation """
+        self._language_options.clear()
+        options = subprocess.check_output(['/usr/share/language-tools/language-options'])
         mylist = []
-        for (i, locale) in enumerate(self._localeinfo.generated_locales()):
-            iter = model.append()
-            model.set(iter,
-                      COMBO_LANGUAGE,self._localeinfo.translate(locale, native=True),
-                      COMBO_CODE, locale)
-            model.set(iter,
-                      LANGTREEVIEW_LANGUAGE,self._localeinfo.translate(locale, native=True),
-                      LANGTREEVIEW_CODE, locale)
-            if (defaultLangName and
-                   self._localeinfo.translate(locale, native=True) == defaultLangName):
-                combo.set_active(i)
-            mylist.append([self._localeinfo.translate(locale, native=True), locale])
-            macr = macros.LangpackMacros(self._datadir, locale)
-            langcode = macr["LCODE"]
-            if '_' in locale and not langcode in installedLangList:
-                installedLangList.append(langcode)
-                mylist.append([self._localeinfo.translate(langcode, native=True), langcode])
+        for (i, option) in enumerate( options.split("\n") ):
+            mylist.append([self._localeinfo.translate(option, native=True), option])
         if len(languageString) > 0:
             self.userEnvLanguage = languageString
             languages = languageString.split(":")
@@ -836,7 +839,17 @@ class GtkLanguageSelector(LanguageSelectorBase,  SimpleGtkbuilderApp):
                 languages = self.userEnvLanguage.split(":")
         mylist_sorted = self.bubbleSort(mylist, languages)
         for i in mylist_sorted:
-            self._localelist.append(i)
+            self._language_options.append(i)
+
+        """ locales for misc. format preferences """
+        for (i, locale) in enumerate( self._localeinfo.generated_locales() ):
+            iter = model.append()
+            model.set_value(iter, LANGTREEVIEW_LANGUAGE,
+                    self._localeinfo.translate(locale, native=True))
+            model.set_value(iter, LANGTREEVIEW_CODE, locale)
+            if (defaultLangName and
+                   self._localeinfo.translate(locale, native=True) == defaultLangName):
+                combo.set_active(i)
         self.updateExampleBox()
 
     def bubbleSort(self, sortlist, presort=None):
@@ -875,7 +888,7 @@ class GtkLanguageSelector(LanguageSelectorBase,  SimpleGtkbuilderApp):
 #        cell = combo.get_child().get_cell_renderers()[0]
 #        # FIXME: use something else than a hardcoded value here
 #        cell.set_property("wrap-width",300)
-#        cell.set_property("wrap-mode",pango.WRAP_WORD)
+#        cell.set_property("wrap-mode",Pango.WRAP_WORD)
 #        model = combo.get_model()
 #        model.clear()
 
@@ -909,7 +922,12 @@ class GtkLanguageSelector(LanguageSelectorBase,  SimpleGtkbuilderApp):
         (lang, code) = model[combo.get_active()]
         macr = macros.LangpackMacros(self._datadir, code)
         mylocale = macr["SYSLOCALE"]
-        locale.setlocale(locale.LC_ALL, mylocale)
+        try:
+            locale.setlocale(locale.LC_ALL, mylocale)
+        except locale.Error as detail:
+            self.label_example_date.set_text('[ '
+            + _("Failed to apply the '%s' format choice:") % mylocale + "\n%s ]" % detail)
+            return
         self.label_example_currency.set_text(locale.currency(20457.99))
         self.label_example_number.set_text(locale.format("%.2f", 1234567.89, grouping=True))
         self.label_example_date.set_text(time.strftime(locale.nl_langinfo(locale.D_T_FMT)))
@@ -918,55 +936,9 @@ class GtkLanguageSelector(LanguageSelectorBase,  SimpleGtkbuilderApp):
     ####################################################
     # window_installer signal handlers                 #
     ####################################################
-    def on_treeview_languages_cursor_changed(self, treeview):
-        #print "on_treeview_languages_cursor_changed()"
-        langInfo = self._get_langinfo_on_cursor()
-        for (button, attr) in ( 
-              ("checkbutton_translations", langInfo.languagePkgList["languagePack"]),
-              ("checkbutton_writing_aids", langInfo.languagePkgList["languageSupportWritingAids"]),
-              ("checkbutton_input_methods", langInfo.languagePkgList["languageSupportInputMethods"]),
-              ("checkbutton_fonts", langInfo.languagePkgList["languageSupportFonts"])  ):
-            self.block_toggle = True
-            if ((attr.installed and not attr.doChange) or (not attr.installed and attr.doChange)) :
-                getattr(self, button).set_active(True)
-            else :
-                getattr(self, button).set_active(False)
-            getattr(self, button).set_sensitive(attr.available)
-            self.block_toggle = False
-            #self.debug_pkg_status()
 
     def on_treeview_languages_row_activated(self, treeview, path, view_column):
-        self.on_toggled(None,str(path[0]))
-
-    # details checkboxes
-    def on_checkbutton_fonts_clicked(self, button):
-        if self.block_toggle: return
-        langInfo = self._get_langinfo_on_cursor()
-        langInfo.languagePkgList["languageSupportFonts"].doChange = not langInfo.languagePkgList["languageSupportFonts"].doChange
-        self.check_status()
-        self.treeview_languages.queue_draw()
-        #self.debug_pkg_status()
-    def on_checkbutton_input_methods_clicked(self, button):
-        if self.block_toggle: return
-        langInfo = self._get_langinfo_on_cursor()
-        langInfo.languagePkgList["languageSupportInputMethods"].doChange = not langInfo.languagePkgList["languageSupportInputMethods"].doChange
-        self.check_status()
-        self.treeview_languages.queue_draw()
-        #self.debug_pkg_status()
-    def on_checkbutton_writing_aids_clicked(self, button):
-        if self.block_toggle: return
-        langInfo = self._get_langinfo_on_cursor()
-        langInfo.languagePkgList["languageSupportWritingAids"].doChange = not langInfo.languagePkgList["languageSupportWritingAids"].doChange
-        self.check_status()
-        self.treeview_languages.queue_draw()
-        #self.debug_pkg_status()
-    def on_checkbutton_translations_clicked(self, button):
-        if self.block_toggle: return
-        langInfo = self._get_langinfo_on_cursor()
-        langInfo.languagePkgList["languagePack"].doChange = not langInfo.languagePkgList["languagePack"].doChange
-        self.check_status()
-        self.treeview_languages.queue_draw()
-        #self.debug_pkg_status()
+        self.on_toggled(None, path.to_string())
 
     # the global toggle
     def on_toggled(self, renderer, path_string):
@@ -1016,17 +988,17 @@ class GtkLanguageSelector(LanguageSelectorBase,  SimpleGtkbuilderApp):
         if self.window_main.get_property("sensitive") is False:
             return True
         else:
-            gtk.main_quit()
+            Gtk.main_quit()
 
     def on_button_quit_clicked(self, widget):
-        gtk.main_quit()
+        Gtk.main_quit()
 
     @honorBlockedSignals
     def on_window_main_key_press_event(self, widget, event):
-        keyname = gtk.gdk.keyval_name(event.keyval)
-        if (event.state & gtk.gdk.CONTROL_MASK):
+        keyname = Gdk.keyval_name(event.keyval)
+        if (event.get_state() & Gdk.ModifierType.CONTROL_MASK):
             if (keyname == "w"):
-                gtk.main_quit()
+                Gtk.main_quit()
         return None
 
     ####################################################
@@ -1058,9 +1030,10 @@ class GtkLanguageSelector(LanguageSelectorBase,  SimpleGtkbuilderApp):
             if str == "en":
                 break
             myiter = model.iter_next(myiter)
-        print (envLanguage)
+        #print (envLanguage)
         self.writeUserLanguage(envLanguage)
         self.userEnvLanguage = envLanguage
+        self.check_input_methods()
         #os.environ["LANGUAGE"]=envLanguage
 
 #    @honorBlockedSignals
@@ -1082,11 +1055,8 @@ class GtkLanguageSelector(LanguageSelectorBase,  SimpleGtkbuilderApp):
 
     @honorBlockedSignals
     def on_combobox_input_method_changed(self, widget):
-        combo = self.combobox_locale_chooser
-        model = combo.get_model()
-        if combo.get_active() < 0:
-            return
-        (lang, code) = model[combo.get_active()]
+        locale = language2locale(self.userEnvLanguage)
+        code = re.split('[.@]', locale)[0]
 
         combo = self.combobox_input_method
         model = combo.get_model()
